@@ -11,21 +11,77 @@ static const char* FILE_NAME = "Sys/IO/Stat.c";
 
 #if defined(_WIN32)
   #include "Sys-Windows.h"
+  
+  // Exactly same as Perl's one in Win32.h
+  #define _S_IFLNK ((unsigned)(_S_IFDIR | _S_IFCHR))
+  
+  // Exactly same as Perl's one in Win32.h
+  #define _S_IFSOCK ((unsigned)(_S_IFDIR | _S_IFIFO))
+  
+  // Exactly same as Perl's one in Win32.h
+  typedef DWORD Dev_t;
+  
+  // Exactly same as Perl's one in Win32.h
+  typedef unsigned __int64 Ino_t;
+  
+  // This is different from Perl's ones, but it must be defined well
+  typedef uint64_t Off_t;
+  
+  // Exactly same as Perl's one in Win32.h
+  struct w32_stat {
+    Dev_t st_dev;
+    Ino_t st_ino;
+    unsigned short st_mode;
+    DWORD st_nlink;
+    short st_uid;
+    short st_gid;
+    Dev_t st_rdev;
+    Off_t st_size;
+    time_t st_atime;
+    time_t st_mtime;
+    time_t st_ctime;
+  };
+  
   typedef struct w32_stat Stat_t;
 #else // defined(_WIN32)
   typedef struct stat Stat_t;
 #endif
 
 #if defined(_WIN32)
-// Same as Perl's one in Win32.c, but this function use Perl data structure SV. I replace it with SPVM data structure.
-// And path argument is not needed.
-static int
-win32_stat_low(HANDLE handle, STRLEN len, Stat_t *sbuf, DWORD reparse_type) {
+
+// The output is the same as Perl's file_time_to_epoch in Win32.c
+static time_t file_time_to_epoch(FILETIME file_time) {
+  SYSTEMTIME system_time;
+  struct tm st_tm = {0};
+  
+  time_t epoch = -1;
+  
+  if (!FileTimeToSystemTime(&file_time, &system_time)) {
+    win_last_error_to_errno();
+    goto END_OF_FUNC;
+  }
+  
+  st_tm.tm_year = system_time.wYear - 1900;
+  st_tm.tm_mon = system_time.wMonth - 1;
+  st_tm.tm_mday = system_time.wDay;
+  st_tm.tm_hour = system_time.wHour;
+  st_tm.tm_min = system_time.wMinute;
+  st_tm.tm_sec = system_time.wSecond;
+  
+  epoch = _mkgmtime(&st_tm);
+  
+  END_OF_FUNC:
+  
+  return epoch;
+}
+
+// The output data is the same as Perl's win32_stat_low in Win32.c.
+static int win_fstat_by_handle(SPVM_ENV* env, SPVM_VALUE* stack, HANDLE handle, int32_t len, Stat_t *sbuf, DWORD reparse_type) {
     DWORD type = GetFileType(handle);
     BY_HANDLE_FILE_INFORMATION bhi;
-
-    Zero(sbuf, 1, Stat_t);
-
+    
+    memset(sbuf, 0, 1 * sizeof(Stat_t));
+    
     if (reparse_type) {
         /* Lie to get to the right place */
         type = FILE_TYPE_DISK;
@@ -50,9 +106,9 @@ win32_stat_low(HANDLE handle, STRLEN len, Stat_t *sbuf, DWORD reparse_type) {
             sbuf->st_size <<= 32;
             sbuf->st_size |= bhi.nFileSizeLow;
 
-            sbuf->st_atime = translate_ft_to_time_t(bhi.ftLastAccessTime);
-            sbuf->st_mtime = translate_ft_to_time_t(bhi.ftLastWriteTime);
-            sbuf->st_ctime = translate_ft_to_time_t(bhi.ftCreationTime);
+            sbuf->st_atime = file_time_to_epoch(bhi.ftLastAccessTime);
+            sbuf->st_mtime = file_time_to_epoch(bhi.ftLastWriteTime);
+            sbuf->st_ctime = file_time_to_epoch(bhi.ftCreationTime);
 
             if (reparse_type) {
                 /* https://docs.microsoft.com/en-us/openspecs/windows_protocols/ms-fscc/c8e77b37-3909-4fe6-a4ea-2b9d423b1ee4
@@ -90,19 +146,15 @@ win32_stat_low(HANDLE handle, STRLEN len, Stat_t *sbuf, DWORD reparse_type) {
                 }
             }
             else {
-                wchar_t path_buf_tmp_w[MAX_PATH+1];
+                WCHAR path_buf_tmp_w[MAX_PATH+1];
                 sbuf->st_mode = _S_IFREG;
                 
                 const char* path = NULL;
                 len = GetFinalPathNameByHandleW(handle, path_buf_tmp_w, sizeof(path_buf_tmp_w), 0);
                 if (len > 0) {
-                  SPVM_ENV* env = thread_env;
-                  
-                  SPVM_VALUE* stack = env->new_stack(env);
-                  
                   int32_t scope_id = env->enter_scope(env, stack);
                   
-                  wchar_t* path_buf_w = env->new_memory_block(env, stack, sizeof(wchar_t) * (len + 1));
+                  WCHAR* path_buf_w = env->new_memory_block(env, stack, sizeof(WCHAR) * (len + 1));
                   
                   len = GetFinalPathNameByHandleW(handle, path_buf_w, len + 1, 0);
                   
@@ -117,8 +169,6 @@ win32_stat_low(HANDLE handle, STRLEN len, Stat_t *sbuf, DWORD reparse_type) {
                   assert(error_id == 0);
                   
                   env->leave_scope(env, stack, scope_id);
-                  
-                  env->free_stack(env, stack);
                 }
 
                 if (path && len > 4 &&
@@ -135,7 +185,7 @@ win32_stat_low(HANDLE handle, STRLEN len, Stat_t *sbuf, DWORD reparse_type) {
             }
         }
         else {
-            translate_to_errno();
+            win_last_error_to_errno();
             return -1;
         }
         break;
@@ -169,7 +219,7 @@ static int32_t win_stat(SPVM_ENV* env, SPVM_VALUE* stack, Stat_t *st_stat) {
   
   const char* path = env->get_chars(env, stack, obj_path);
   
-  wchar_t* path_w = utf8_to_win_wchar(env, stack, path, &error_id, __func__, FILE_NAME, __LINE__);
+  WCHAR* path_w = utf8_to_win_wchar(env, stack, path, &error_id, __func__, FILE_NAME, __LINE__);
   if (error_id) {
     return error_id;
   }
@@ -193,7 +243,7 @@ static int32_t win_stat(SPVM_ENV* env, SPVM_VALUE* stack, Stat_t *st_stat) {
     }
     const char* resolved_link_text = env->get_chars(env, stack, obj_resolved_link_text);
     
-    wchar_t* resolved_link_text_w = utf8_to_win_wchar(env, stack, resolved_link_text, &error_id, __func__, FILE_NAME, __LINE__);
+    WCHAR* resolved_link_text_w = utf8_to_win_wchar(env, stack, resolved_link_text, &error_id, __func__, FILE_NAME, __LINE__);
     if (error_id) {
       return error_id;
     }
@@ -201,7 +251,7 @@ static int32_t win_stat(SPVM_ENV* env, SPVM_VALUE* stack, Stat_t *st_stat) {
     handle = CreateFileW_reparse_point_for_read(resolved_link_text_w);
     
     if (handle == INVALID_HANDLE_VALUE) {
-      translate_to_errno();
+      win_last_error_to_errno();
       error_id = SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_SYSTEM_CLASS;
       goto END_OF_FUNC;
     }
@@ -214,7 +264,7 @@ static int32_t win_stat(SPVM_ENV* env, SPVM_VALUE* stack, Stat_t *st_stat) {
     }
   }
   
-  int32_t result = win32_stat_low(handle, 0, st_stat, ReparseTag);
+  int32_t result = win_fstat_by_handle(env, stack, handle, 0, st_stat, ReparseTag);
   
   if (result == -1) {
     error_id = SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_SYSTEM_CLASS;
@@ -229,7 +279,7 @@ static int32_t win_stat(SPVM_ENV* env, SPVM_VALUE* stack, Stat_t *st_stat) {
   
   if (error_id) {
     if (errno) {
-      env->die(env, stack, "[System Error]win_stat() failed:%s. $path:\"%s\".", env->strerror_nolen(env, stack, errno), path, __func__, FILE_NAME, __LINE__);
+      env->die(env, stack, "[System Error]win_stat() failed(%d: %s). $path:\"%s\".", errno, env->strerror_nolen(env, stack, errno), path, __func__, FILE_NAME, __LINE__);
     }
     
     return error_id;
@@ -246,26 +296,26 @@ static int32_t win_lstat(SPVM_ENV* env, SPVM_VALUE* stack, Stat_t *st_stat) {
   
   const char* path = env->get_chars(env, stack, obj_path);
   
-  wchar_t* path_w = utf8_to_win_wchar(env, stack, path, &error_id, __func__, FILE_NAME, __LINE__);
+  WCHAR* path_w = utf8_to_win_wchar(env, stack, path, &error_id, __func__, FILE_NAME, __LINE__);
   if (error_id) {
     goto END_OF_FUNC;
   }
   
   HANDLE handle = CreateFileW_reparse_point_for_read(path_w);
   if (handle == INVALID_HANDLE_VALUE) {
-    translate_to_errno();
+    win_last_error_to_errno();
     error_id = SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_SYSTEM_CLASS;
     goto END_OF_FUNC;
   }
   
-  int32_t result = win32_stat_low(handle, 0, st_stat, 0);
+  int32_t result = win_fstat_by_handle(env, stack, handle, 0, st_stat, 0);
   
   if (result == -1) {
     error_id = SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_SYSTEM_CLASS;
     goto END_OF_FUNC;
   }
   
-  int32_t is_sym = is_symlink(handle);
+  int32_t is_sym = is_symlink_by_handle(handle);
   
   if (is_sym) {
     void* obj_link_text = NULL;
@@ -290,7 +340,7 @@ static int32_t win_lstat(SPVM_ENV* env, SPVM_VALUE* stack, Stat_t *st_stat) {
   
   if (error_id) {
     if (errno) {
-      env->die(env, stack, "[System Error]win_lstat() failed:%s. $path:\"%s\".", env->strerror_nolen(env, stack, errno), path, __func__, FILE_NAME, __LINE__);
+      env->die(env, stack, "[System Error]win_lstat() failed(%d: %s). $path:\"%s\".", errno, env->strerror_nolen(env, stack, errno), path, __func__, FILE_NAME, __LINE__);
     }
     
     return error_id;
@@ -350,7 +400,6 @@ int32_t SPVM__Sys__IO__Stat__stat(SPVM_ENV* env, SPVM_VALUE* stack) {
   Stat_t* stat_buf = env->get_pointer(env, stack, obj_stat);
   
 #if defined(_WIN32)
-  thread_env = env;
   stack[0].oval = obj_path;
   error_id = win_stat(env, stack, stat_buf);
   int32_t status = error_id ? -1 : 0;
@@ -360,7 +409,7 @@ int32_t SPVM__Sys__IO__Stat__stat(SPVM_ENV* env, SPVM_VALUE* stack) {
   
   if (status == -1) {
     const char* path = env->get_chars(env, stack, obj_path);
-    env->die(env, stack, "[System Error]stat() failed:%s. $path:\"%s\".", env->strerror_nolen(env, stack, errno), path, __func__, FILE_NAME, __LINE__);
+    env->die(env, stack, "[System Error]stat() failed(%d: %s). $path:\"%s\".", errno, env->strerror_nolen(env, stack, errno), path, __func__, FILE_NAME, __LINE__);
     return SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_SYSTEM_CLASS;
   }
   
@@ -389,7 +438,6 @@ int32_t SPVM__Sys__IO__Stat__lstat(SPVM_ENV* env, SPVM_VALUE* stack) {
   Stat_t* stat_buf = env->get_pointer(env, stack, obj_lstat);
   
 #if defined(_WIN32)
-  thread_env = env;
   stack[0].oval = obj_path;
   error_id = win_lstat(env, stack, stat_buf);
   int32_t status = error_id ? -1 : 0;
@@ -399,7 +447,7 @@ int32_t SPVM__Sys__IO__Stat__lstat(SPVM_ENV* env, SPVM_VALUE* stack) {
 
   if (status == -1) {
     const char* path = env->get_chars(env, stack, obj_path);
-    env->die(env, stack, "[System Error]lstat() failed:%s. $path:\"%s\".", env->strerror_nolen(env, stack, errno), path, __func__, FILE_NAME, __LINE__);
+    env->die(env, stack, "[System Error]lstat() failed(%d: %s). $path:\"%s\".", errno, env->strerror_nolen(env, stack, errno), path, __func__, FILE_NAME, __LINE__);
     return SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_SYSTEM_CLASS;
   }
   
@@ -424,15 +472,14 @@ int32_t SPVM__Sys__IO__Stat__fstat(SPVM_ENV* env, SPVM_VALUE* stack) {
   
   
 #if defined(_WIN32)
-  thread_env = env;
-  HANDLE handle = (HANDLE)win32_get_osfhandle(fd);
-  int32_t status = win32_stat_low(handle, 0, stat_buf, 0);
+  HANDLE handle = (HANDLE)_get_osfhandle(fd);
+  int32_t status = win_fstat_by_handle(env, stack, handle, 0, stat_buf, 0);
 #else
   int32_t status = fstat(fd, stat_buf);
 #endif
 
   if (status == -1) {
-    env->die(env, stack, "[System Error]fstat() failed:%s.", env->strerror_nolen(env, stack, errno), __func__, FILE_NAME, __LINE__);
+    env->die(env, stack, "[System Error]fstat() failed(%d: %s).", errno, env->strerror_nolen(env, stack, errno), __func__, FILE_NAME, __LINE__);
     return SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_SYSTEM_CLASS;
   }
   
