@@ -7,115 +7,100 @@ static const char* FILE_NAME = "Sys/IO/Windows.c";
 
 #if defined(_WIN32)
 
-// These implementations are originally copied form Perl win32/win32.c and win32/win32.h
-
 #include "Sys-Windows.h"
 
-static int
-win32_symlink(SPVM_ENV* env, SPVM_VALUE* stack, const wchar_t *oldfile, const wchar_t *newfile)
-{
-    size_t oldfile_len = wcslen(oldfile);
-    pCreateSymbolicLinkW_t pCreateSymbolicLinkW =
-        (pCreateSymbolicLinkW_t)GetProcAddress(GetModuleHandle("kernel32.dll"), "CreateSymbolicLinkW");
-    DWORD create_flags = 0;
+#define isSLASHW(c) ((c) == L'/' || (c) == L'\\')
+#define strEQW(string1, string2) (wcscmp(string1, string2) == 0)
 
-    /* this flag can be used only on Windows 10 1703 or newer */
-    if (g_osver.dwMajorVersion > 10 ||
-        (g_osver.dwMajorVersion == 10 &&
-         (g_osver.dwMinorVersion > 0 || g_osver.dwBuildNumber > 15063)))
-    {
-        create_flags |= SYMBOLIC_LINK_FLAG_ALLOW_UNPRIVILEGED_CREATE;
-    }
-
-    if (!pCreateSymbolicLinkW) {
-        errno = ENOSYS;
-        return -1;
-    }
-
-    if (wcschr(oldfile, L'/')) {
-        /* Win32 (or perhaps NTFS) won't follow symlinks containing
-           /, so replace any with \\
-        */
-        wchar_t *temp = (wchar_t*)env->new_memory_block(env, stack, (wcslen(oldfile) + 1) * sizeof(wchar_t));
-        memcpy(temp, oldfile, (wcslen(oldfile) + 1) * sizeof(wchar_t));
-        wchar_t *p = temp;
-        while (*p) {
-            if (*p == L'/') {
-                *p = '\\';
-            }
-            ++p;
-        }
-        *p = 0;
-        oldfile = temp;
-        oldfile_len = p - temp;
-        env->free_memory_block(env, stack, temp);
-    }
-
-    /* are we linking to a directory?
-       CreateSymlinkW() needs to know if the target is a directory,
-       If it looks like a directory name:
-        - ends in slash
-        - is just . or ..
-        - ends in /. or /.. (with either slash)
-        - is a simple drive letter
-       assume it's a directory.
-       Otherwise if the oldfile is relative we need to make a relative path
-       based on the newfile to check if the target is a directory.
+// The logic is the same as Perl's win32_symlink in Win32.c, and supports UTF-8 arugments.
+static int win32_symlink(SPVM_ENV* env, SPVM_VALUE* stack, const WCHAR *oldpath_w, const WCHAR *newpath_w) {
+  DWORD create_flags = SYMBOLIC_LINK_FLAG_ALLOW_UNPRIVILEGED_CREATE;
+  size_t oldpath_w_len = wcslen(oldpath_w);
+  
+  if (wcschr(oldpath_w, L'/')) {
+    /* Win32 (or perhaps NTFS) won't follow symlinks containing
+       /, so replace any with \\
     */
-    if ((oldfile_len >= 1 && isSLASHW(oldfile[oldfile_len-1])) ||
-        strEQW(oldfile, L"..") ||
-        strEQW(oldfile, L".") ||
-        (isSLASH(oldfile[oldfile_len-2]) && oldfile[oldfile_len-1] == '.') ||
-        strEQW(oldfile+oldfile_len-3, L"\\..") ||
-        (oldfile_len == 2 && oldfile[1] == L':')) {
-        create_flags |= SYMBOLIC_LINK_FLAG_DIRECTORY;
+    WCHAR *temp = (WCHAR*)env->new_memory_block(env, stack, (wcslen(oldpath_w) + 1) * sizeof(WCHAR));
+    memcpy(temp, oldpath_w, (wcslen(oldpath_w) + 1) * sizeof(WCHAR));
+    WCHAR *p = temp;
+    while (*p) {
+      if (*p == L'/') {
+        *p = L'\\';
+      }
+      ++p;
     }
-    else {
-        DWORD dest_attr;
-        const wchar_t *dest_path = oldfile;
-        wchar_t szTargetName[MAX_PATH+1];
-
-        if (oldfile_len >= 3 && oldfile[1] == ':') {
-            /* relative to current directory on a drive, or absolute */
-            /* dest_path = oldfile; already done */
-        }
-        else if (oldfile[0] != L'\\') {
-            size_t newfile_len = wcslen(newfile);
-            wchar_t *last_slash = wcsrchr(newfile, L'/');
-            wchar_t *last_bslash = wcsrchr(newfile, L'\\');
-            wchar_t *end_dir = last_slash && last_bslash
-                ? ( last_slash > last_bslash ? last_slash : last_bslash)
-                : last_slash ? last_slash : last_bslash ? last_bslash : NULL;
-
-            if (end_dir) {
-                if ((end_dir - newfile + 1) + oldfile_len > MAX_PATH) {
-                    /* too long */
-                    errno = EINVAL;
-                    return -1;
-                }
-
-                memcpy(szTargetName, newfile, (end_dir - newfile + 1) * sizeof(wchar_t));
-                wcscpy(szTargetName + (end_dir - newfile + 1), oldfile);
-                dest_path = szTargetName;
-            }
-            else {
-                /* newpath is just a filename */
-                /* dest_path = oldfile; */
-            }
-        }
-
-        dest_attr = GetFileAttributesW(dest_path);
-        if (dest_attr != (DWORD)-1 && (dest_attr & FILE_ATTRIBUTE_DIRECTORY)) {
-            create_flags |= SYMBOLIC_LINK_FLAG_DIRECTORY;
-        }
+    *p = 0;
+    oldpath_w = temp;
+    oldpath_w_len = p - temp;
+    env->free_memory_block(env, stack, temp);
+  }
+  
+  /* are we linking to a directory?
+     CreateSymlinkW() needs to know if the target is a directory,
+     If it looks like a directory name:
+      - ends in slash
+      - is just . or ..
+      - ends in /. or /.. (with either slash)
+      - is a simple drive letter
+     assume it's a directory.
+     Otherwise if the oldpath_w is relative we need to make a relative path
+     based on the newpath_w to check if the target is a directory.
+  */
+  if ((oldpath_w_len >= 1 && isSLASHW(oldpath_w[oldpath_w_len - 1])) ||
+    strEQW(oldpath_w, L"..") ||
+    strEQW(oldpath_w, L".") ||
+    (isSLASHW(oldpath_w[oldpath_w_len-2]) && oldpath_w[oldpath_w_len - 1] == L'.') ||
+    strEQW(oldpath_w+oldpath_w_len-3, L"\\..") ||
+    (oldpath_w_len == 2 && oldpath_w[1] == L':')) {
+    create_flags |= SYMBOLIC_LINK_FLAG_DIRECTORY;
+  }
+  else {
+    DWORD dest_attr;
+    const WCHAR *dest_path_w = oldpath_w;
+    WCHAR target_name_w[MAX_PATH+1];
+    
+    if (oldpath_w_len >= 3 && oldpath_w[1] == L':') {
+      /* relative to current directory on a drive, or absolute */
+      /* dest_path_w = oldpath_w; already done */
     }
-
-    if (!pCreateSymbolicLinkW(newfile, oldfile, create_flags)) {
-        translate_to_errno();
-        return -1;
+    else if (oldpath_w[0] != L'\\') {
+      size_t newpath_w_len = wcslen(newpath_w);
+      WCHAR *last_slash_w = wcsrchr(newpath_w, L'/');
+      WCHAR *last_bslash_w = wcsrchr(newpath_w, L'\\');
+      WCHAR *end_dir_w = last_slash_w && last_bslash_w
+        ? ( last_slash_w > last_bslash_w ? last_slash_w : last_bslash_w)
+        : last_slash_w ? last_slash_w : last_bslash_w ? last_bslash_w : NULL;
+      
+      if (end_dir_w) {
+        if ((end_dir_w - newpath_w + 1) + oldpath_w_len > MAX_PATH) {
+          /* too long */
+          errno = EINVAL;
+          return -1;
+        }
+        
+        memcpy(target_name_w, newpath_w, (end_dir_w - newpath_w + 1) * sizeof(WCHAR));
+        wcscpy(target_name_w + (end_dir_w - newpath_w + 1), oldpath_w);
+        dest_path_w = target_name_w;
+      }
+      else {
+        /* newpath_w is just a filename */
+        /* dest_path_w = oldpath_w; */
+      }
     }
-
-    return 0;
+    
+    dest_attr = GetFileAttributesW(dest_path_w);
+    if (dest_attr != (DWORD)-1 && (dest_attr & FILE_ATTRIBUTE_DIRECTORY)) {
+      create_flags |= SYMBOLIC_LINK_FLAG_DIRECTORY;
+    }
+  }
+  
+  if (!CreateSymbolicLinkW(newpath_w, oldpath_w, create_flags)) {
+    win_last_error_to_errno();
+    return -1;
+  }
+  
+  return 0;
 }
 
 #endif // _WIN32
@@ -137,7 +122,7 @@ int32_t SPVM__Sys__IO__Windows__unlink(SPVM_ENV* env, SPVM_VALUE* stack) {
   
   const char* path = env->get_chars(env, stack, obj_path);
   
-  wchar_t* path_w = utf8_to_win_wchar(env, stack, path, &error_id, __func__, FILE_NAME, __LINE__);
+  WCHAR* path_w = utf8_to_win_wchar(env, stack, path, &error_id, __func__, FILE_NAME, __LINE__);
   if (error_id) {
     return error_id;
   }
@@ -161,7 +146,7 @@ int32_t SPVM__Sys__IO__Windows__unlink(SPVM_ENV* env, SPVM_VALUE* stack) {
   }
   else if ((attrs & (FILE_ATTRIBUTE_REPARSE_POINT | FILE_ATTRIBUTE_DIRECTORY))
     == (FILE_ATTRIBUTE_REPARSE_POINT | FILE_ATTRIBUTE_DIRECTORY)
-         && is_symlink_name(path_w))
+         && is_symlink(path_w))
   {
     status = _wrmdir(path_w);
   }
@@ -203,12 +188,12 @@ int32_t SPVM__Sys__IO__Windows__rename(SPVM_ENV* env, SPVM_VALUE* stack) {
   }
   const char* newpath = env->get_chars(env, stack, obj_newpath);
   
-  wchar_t* oldpath_w = utf8_to_win_wchar(env, stack, oldpath, &error_id, __func__, FILE_NAME, __LINE__);
+  WCHAR* oldpath_w = utf8_to_win_wchar(env, stack, oldpath, &error_id, __func__, FILE_NAME, __LINE__);
   if (error_id) {
     return error_id;
   }
   
-  wchar_t* newpath_w = utf8_to_win_wchar(env, stack, newpath, &error_id, __func__, FILE_NAME, __LINE__);
+  WCHAR* newpath_w = utf8_to_win_wchar(env, stack, newpath, &error_id, __func__, FILE_NAME, __LINE__);
   if (error_id) {
     return error_id;
   }
@@ -278,13 +263,13 @@ int32_t SPVM__Sys__IO__Windows__win_readlink(SPVM_ENV* env, SPVM_VALUE* stack) {
   }
   const char* path = env->get_chars(env, stack, obj_path);
   
-  wchar_t* path_w = utf8_to_win_wchar(env, stack, path, &error_id, __func__, FILE_NAME, __LINE__);
+  WCHAR* path_w = utf8_to_win_wchar(env, stack, path, &error_id, __func__, FILE_NAME, __LINE__);
   if (error_id) {
     goto END_OF_FUNC;
   }
   DWORD fileattr = GetFileAttributesW(path_w);
   if (fileattr == INVALID_FILE_ATTRIBUTES) {
-    translate_to_errno();
+    win_last_error_to_errno();
     env->die(env, stack, "[System Error]GetFileAttributesW() failed(%d: %s). $path=\"%s\".", errno, env->strerror_nolen(env, stack, errno), path, __func__, FILE_NAME, __LINE__);    
     error_id = SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_SYSTEM_CLASS;
     goto END_OF_FUNC;
@@ -297,14 +282,14 @@ int32_t SPVM__Sys__IO__Windows__win_readlink(SPVM_ENV* env, SPVM_VALUE* stack) {
     handle = CreateFileW_reparse_point_for_read(path_w);
     
     if (handle == INVALID_HANDLE_VALUE) {
-      translate_to_errno();
+      win_last_error_to_errno();
       env->die(env, stack, "[System Error]CreateFileW() failed when opening a file(%d: %s). $path=\"%s\".", errno, env->strerror_nolen(env, stack, errno), path, __func__, FILE_NAME, __LINE__);
       error_id = SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_SYSTEM_CLASS;
       goto END_OF_FUNC;
     }
     
     if (!DeviceIoControl(handle, FSCTL_GET_REPARSE_POINT, NULL, 0, &linkdata, sizeof(linkdata), &linkdata_returned, NULL)) {
-      translate_to_errno();
+      win_last_error_to_errno();
       env->die(env, stack, "[System Error]DeviceIoControl() failed(%d: %s). $path=\"%s\".", errno, env->strerror_nolen(env, stack, errno), path, __func__, FILE_NAME, __LINE__);
       error_id = SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_SYSTEM_CLASS;
       goto END_OF_FUNC;
@@ -314,7 +299,7 @@ int32_t SPVM__Sys__IO__Windows__win_readlink(SPVM_ENV* env, SPVM_VALUE* stack) {
     handle = CreateFileW_for_read(path_w);
     
     if (handle == INVALID_HANDLE_VALUE) {
-      translate_to_errno();
+      win_last_error_to_errno();
       env->die(env, stack, "[System Error]CreateFileW() failed when opening a file(%d: %s). $path=\"%s\".", errno, env->strerror_nolen(env, stack, errno), path, __func__, FILE_NAME, __LINE__);
       error_id = SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_SYSTEM_CLASS;
       goto END_OF_FUNC;
@@ -327,7 +312,7 @@ int32_t SPVM__Sys__IO__Windows__win_readlink(SPVM_ENV* env, SPVM_VALUE* stack) {
     }
   }
   
-  const wchar_t* PathBuffer = NULL;
+  const WCHAR* PathBuffer = NULL;
   int32_t PrintNameOffset = -1;
   int32_t PrintNameLength = -1;
   switch (linkdata.ReparseTag) {
@@ -435,12 +420,12 @@ int32_t SPVM__Sys__IO__Windows__symlink(SPVM_ENV* env, SPVM_VALUE* stack) {
   }
   const char* newpath = env->get_chars(env, stack, obj_newpath);
   
-  wchar_t* oldpath_w = utf8_to_win_wchar(env, stack, oldpath, &error_id, __func__, FILE_NAME, __LINE__);
+  WCHAR* oldpath_w = utf8_to_win_wchar(env, stack, oldpath, &error_id, __func__, FILE_NAME, __LINE__);
   if (error_id) {
     return error_id;
   }
   
-  wchar_t* newpath_w = utf8_to_win_wchar(env, stack, newpath, &error_id, __func__, FILE_NAME, __LINE__);
+  WCHAR* newpath_w = utf8_to_win_wchar(env, stack, newpath, &error_id, __func__, FILE_NAME, __LINE__);
   if (error_id) {
     return error_id;
   }
@@ -449,7 +434,7 @@ int32_t SPVM__Sys__IO__Windows__symlink(SPVM_ENV* env, SPVM_VALUE* stack) {
   int32_t status = win32_symlink(env, stack, oldpath_w, newpath_w);
   
   if (status == -1) {
-    env->die(env, stack, "[System Error]win32_symlink() failed:%s. $oldpath is \"%s\". $newpath is \"%s\".", env->strerror_nolen(env, stack, errno), oldpath, newpath, __func__, FILE_NAME, __LINE__);
+    env->die(env, stack, "[System Error]win32_symlink() failed(%d: %s). $oldpath=\"%s\", $newpath=\"%s\".", errno, env->strerror_nolen(env, stack, errno), oldpath, newpath, __func__, FILE_NAME, __LINE__);
     return SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_SYSTEM_CLASS;
   }
   
@@ -492,7 +477,7 @@ int32_t SPVM__Sys__IO__Windows__realpath(SPVM_ENV* env, SPVM_VALUE* stack) {
   }
   const char* resolved_link_text = env->get_chars(env, stack, obj_resolved_link_text);
   
-  wchar_t* resolved_link_text_w = utf8_to_win_wchar(env, stack, resolved_link_text, &error_id, __func__, FILE_NAME, __LINE__);
+  WCHAR* resolved_link_text_w = utf8_to_win_wchar(env, stack, resolved_link_text, &error_id, __func__, FILE_NAME, __LINE__);
   if (error_id) {
     return error_id;
   }
@@ -500,7 +485,7 @@ int32_t SPVM__Sys__IO__Windows__realpath(SPVM_ENV* env, SPVM_VALUE* stack) {
   HANDLE handle = CreateFileW_reparse_point_for_read(resolved_link_text_w);
   
   if (handle == INVALID_HANDLE_VALUE) {
-    translate_to_errno();
+    win_last_error_to_errno();
     error_id = SPVM_NATIVE_C_BASIC_TYPE_ID_ERROR_SYSTEM_CLASS;
     goto END_OF_FUNC;
   }
@@ -514,7 +499,7 @@ int32_t SPVM__Sys__IO__Windows__realpath(SPVM_ENV* env, SPVM_VALUE* stack) {
   }
   
   void* obj_resolved_path_w = env->new_short_array(env, stack, needed_len);
-  wchar_t* resolved_path_w = (wchar_t*)env->get_elems_short(env, stack, obj_resolved_path_w);
+  WCHAR* resolved_path_w = (WCHAR*)env->get_elems_short(env, stack, obj_resolved_path_w);
   
   int32_t len = GetFinalPathNameByHandleW(handle, resolved_path_w, needed_len, 0);
   
@@ -561,7 +546,7 @@ int32_t SPVM__Sys__IO__Windows__realpath(SPVM_ENV* env, SPVM_VALUE* stack) {
   
   if (error_id) {
     if (errno) {
-      env->die(env, stack, "[System Error]win_realpath() failed:%s. $path=\"%s\".", env->strerror_nolen(env, stack, errno), path, __func__, FILE_NAME, __LINE__);
+      env->die(env, stack, "[System Error]win_realpath() failed(%d: %s). $path=\"%s\".", errno, env->strerror_nolen(env, stack, errno), path, __func__, FILE_NAME, __LINE__);
     }
     
     return error_id;
@@ -588,12 +573,12 @@ int32_t SPVM__Sys__IO__Windows__is_symlink(SPVM_ENV* env, SPVM_VALUE* stack) {
   
   const char* path = env->get_chars(env, stack, obj_path);
   
-  wchar_t* path_w = utf8_to_win_wchar(env, stack, path, &error_id, __func__, FILE_NAME, __LINE__);
+  WCHAR* path_w = utf8_to_win_wchar(env, stack, path, &error_id, __func__, FILE_NAME, __LINE__);
   if (error_id) {
     return error_id;
   }
   
-  int32_t ret = is_symlink_name(path_w);
+  int32_t ret = is_symlink(path_w);
   
   stack[0].ival = ret;
   
